# Peer@DC 
print('starting script. please wait...')

from pandas import read_excel, isna
from json import load,dumps
from numpy import array
from jsonschema import validate


def write_dict_to_json(data, path):
    '''
    Parameters:
        data: dictionary
        path: directory and name, where the json will be saved
    '''
    j = dumps(data, indent=4)
    f = open(path, 'w')
    print(j, file=f)
    f.close()
    return
    
def sort_public(data_dict):
    new_dict={}
    public={}
    confid={}
    for key in data_dict['public']:
        public[key]=data_dict[key]
    for key in data_dict['confidential']:
        confid[key]=data_dict[key]
    new_dict['public'] = public
    new_dict['confidential']=confid
    return new_dict

def dc_plausibility(dc_annual):
    check = {}
    try:    
        check['check_renewable'] = [dc_annual['kpi_ref'] / dc_annual['el_supply_share_renewables'], 'ref/el_share_renewables']
        check['check_dc_el_sum'] = [dc_annual['el_total_dc_kwh'] / (dc_annual['el_production_kwh']+dc_annual['el_grid_kwh']-dc_annual['el_feed_kwh']), 'el_total_dc_kwh/(el_production_kwh+el_grid_kwh-el_feed_kwh)']
        check['check_dc_el'] = [dc_annual['el_total_dc_kwh'] / (dc_annual['el_it_kwh'] + dc_annual['el_heatp_kwh']+ dc_annual['el_cooling_kwh']), 'el_total_dc_kwh/(el_it_kwh+el_heatp_kwh+el_cooling_kwh)']
        check['check_heat'] = [(dc_annual['el_it_kwh'] + dc_annual['el_cooling_kwh'] ) / (dc_annual['heat_reuse_kwh'] + dc_annual['heat_unuse_kwh']), '(el_it_kwh+el_cooling_kwh)/(heat_reuse_kwh+heat_unuse_kwh)']
    except Exception as e:
        print('Minor error: Plausibility could not be checked',e)
    return check

def validate_dict(operator,which_data, data_dict):
    print('Validating data of: ',operator,which_data)
    with open(f'schemata/peerdc_{operator}_{which_data}_schema.json', 'r') as f:
        schema = load(f)
    try:
        validate(data_dict, schema=schema)
        success=True
    except Exception as e:
        print(f'Major Error: Validation of {operator}_{which_data} was NOT successful! Please make sure that all mandatory fields are filled in correctly. The following error message might give you a hint, where the error occured. \n',e)
        success=False
    return success

def dataframe_to_dict(dtframe):
    '''
    Parameters:
        dataframe (pandas Dataframe)
    Returns:
        dictionary {variable_name : variable, ..., confidential:[...], public:[...]}
    '''
    data = {}
    public = []
    confidential=[]
    for i in range(len(dtframe)):
        variable=dtframe['Data'][i]
        if isna(variable):
            variable=None
        elif variable=='true':
            variable = True
        elif variable=='false':
            variable= False
        elif dtframe['data type'][i] == 'string':
            variable = str(dtframe['Data'][i])
        elif dtframe['data type'][i] == 'string' and variable==0:
            variable = ''
        elif dtframe['data type'][i] == 'number' or dtframe['data type'][i] == 'number,null':
            try:
                variable = float(str(variable).replace(',','.'))
            except ValueError:
                print('Error at '+dtframe['English'][i],': ',dtframe['Data'][i],' is not a number. Please correct your input data in the Excel sheet, save and retry')
                print(ValueError)
        elif dtframe['data type'][i] == 'integer' or dtframe['data type'][i] == 'integer,null':
            try:
                variable = int(variable)
            except ValueError:
                print('Error at '+dtframe['English'][i],': ',dtframe['Data'][i],' is not an integer. Please correct your input data in the Excel sheet, save and retry')
                print(ValueError)
        elif dtframe['data type'][i] == 'array' or dtframe['data type'][i]=='array,null':
            variable=str(dtframe['Data'][i]).replace(',','.').split(';')
            if dtframe['items type'][i]=='number':
                try:
                    variable = array(variable, dtype=float).tolist()
                except ValueError:
                    print('Error at '+dtframe['English'][i],': ',dtframe['Data'][i],' is not a an array of comma-seperated numbers. Please correct your input data in the Excel sheet, save and retry')
            elif dtframe['items type'][i] == 'integer':
                try:
                    variable = array(variable, dtype=int).tolist()
                except ValueError:
                    print('Error at '+dtframe['English'][i],': ',dtframe['Data'][i],' is not a an array of comma-seperated numbers. Please correct your input data in the Excel sheet, save and retry')
            else: 
                pass
        else:
            pass
        data[dtframe["variable name"][i]] = variable
        if dtframe['public'][i]=='ö':
            public.append(dtframe["variable name"][i])
        else:
            confidential.append(dtframe["variable name"][i])
    data['public']= public
    data['confidential'] = confidential
    return data

def convert(operator,which_data,path):
    data_dict = dataframe_to_dict(read_excel(path, sheet_name=f'{operator}_{which_data}', skiprows=6))
    if data_dict['export_me']==True:
        success = validate_dict(operator,which_data, data_dict)
        if success:
            if operator!='DC'or which_data!='annual':
                data_dict=sort_public(data_dict)
            else:
                sorted_dict = sort_public(data_dict)
                sorted_dict['check'] = dc_plausibility(data_dict)
                data_dict=sorted_dict
            data_dict['validation_success']=True
            print('Data of '+f'{operator}_{which_data} is valid.')
            save = True
        else:
            if input('Data of '+f'{operator}_{which_data} is NOT valid. \n'+f'Would you like to write {operator}_{which_data} to json anyways? Confirm with [y]: ' )== 'y':
                data_dict=sort_public(data_dict)
                data_dict['validation_success']=False
                save=True
            else:
                save=False
    else:
        save=False
        data_dict=None
    return save, data_dict

def ask_which_data():
    path = input('The default path to data source is "PEER_DC-Erfassungsbogen_RZ-und-IT-v01.xlsx". \nPress [y] for yes or enter the correct path: ')
    if path == 'y':
        path = "../PEER_DC-Erfassungsbogen_RZ-und-IT-v01.xlsx"
    return path

path = ask_which_data()
save_these = []
data = {}
for operator in ['DC','IT']:
    for data_type in ['general','annual']:
        save, data_dict = convert(operator, data_type, path)
        if save:
            save_these.append([operator,data_type])
            data[f'{operator}_{data_type}'] = data_dict

write_dict_to_json(data,'../export.json')


for i in save_these:
    print(f'{i[0]}_{i[1]}.')
print('were successfully written to export.json')